import logging

from .runner import StatumRunner
from .courses import COORDINATES

MIN_SPEED = 5
MAX_SPEED = 40
MAX_LATENCY = 5

logger = logging.getLogger(__name__)


class StatumMarket:
    def __init__(
        self,
        market_id,
        course,
        race_type,
        surface_type=None,
        race_distance=None,
        coordinates=COORDINATES,
        max_latency=MAX_LATENCY,
        min_speed=MIN_SPEED,
        max_speed=MAX_SPEED,
    ):
        self.market_id = market_id
        self.course = course
        self._race_type = race_type
        self.surface_type = surface_type
        self.race_distance = race_distance  # todo
        self.max_latency = max_latency
        self.min_speed = min_speed
        self.max_speed = max_speed
        self.active = True

        try:
            if surface_type is None:
                self.surface_type = coordinates[self.course][self.race_type][
                    "surface_default"
                ]
            _coordinates = coordinates[self.course][self.race_type][self.surface_type]
            self._finish_straight_distance = _coordinates["finish_straight_distance"]
        except KeyError:
            logger.warning("Coordinates not available for {0}".format(self.market_id))
            self._finish_straight_distance = None

        self.race_progress = None
        self.race_runners = {}

    def __call__(self, race_update, market_book):
        _current_active = self.active
        self.active = True  # reset every update
        self.race_progress = race_update.race_progress

        if isinstance(market_book, dict):
            runner_dict = {
                runner["selectionId"]: runner for runner in market_book["runners"]
            }
        else:
            runner_dict = {
                runner.selection_id: runner for runner in market_book.runners
            }

        for race_change in race_update.race_runners:
            selection_id = race_change.selection_id
            try:
                runner_book = runner_dict[selection_id]
            except KeyError:
                continue  # Bug where MarketBook.runners may not contain a removed runner
            if selection_id not in self.race_runners:
                self.race_runners[selection_id] = StatumRunner(
                    race_change, self.min_speed, self.max_speed
                )

            race_runner = self.race_runners[selection_id]
            race_runner(race_change, runner_book)
            race_runner.check_active()
            if (
                race_runner.active
                and not race_runner.ignore_runner
                and race_runner.feed_latency > self.max_latency
            ):
                self.active = False
                if _current_active is True:
                    logger.warning(
                        "Market {0} made inactive due to latency issue in runner: {1}".format(
                            self.market_id, selection_id
                        )
                    )
        if _current_active is False and self.active is True:
            logger.warning("Market {0} made active again".format(self.market_id))

    """ race """

    @property
    def race_running_time(self):
        """Running time in seconds since
        start"""
        if self.race_progress:
            return self.race_progress.running_time

    @property
    def race_running_time_left(self):
        """Predicted time left in seconds
        based on speed and distance left
        """
        if self.race_progress:
            try:
                return self.race_distance_left / self.race_progress.speed
            except ZeroDivisionError:
                return

    @property
    def race_distance_left(self):
        """Race distance left in metres"""
        if self.race_progress:
            return self.race_progress.progress

    @property
    def race_on_finish_straight(self):
        if self.race_progress and self._finish_straight_distance:
            if 0 < self.race_distance_left < self._finish_straight_distance:
                return True
        return False

    """ runner """

    def runner_distance_to_finish(self, selection_id):
        """Runner distance from finishing
        line"""
        runner = self.race_runners[selection_id]
        return runner.progress

    def runner_seconds_to_finish(self, selection_id):
        """Runner predicted seconds to
        finish based on distance to
        finish and current speed"""
        runner = self.race_runners[selection_id]
        return runner.progress_seconds_to_finish

    def runner_position(self, selection_id):
        """Runner position based on race
        data"""
        return self.race_progress.order.index(selection_id)

    def runner_position_calculated(self, selection_id):
        position = []  # (selection_id, distance_to_finish)
        for _selection_id, runner in self.race_runners.items():
            position.append((_selection_id, runner.runner_distance_to_finish))
        position = sorted(position, key=lambda tup: tup[1])  # sort
        position_sel = [i[0] for i in position]
        try:
            return position_sel.index(selection_id)
        except ValueError:
            return

    """ progress """

    def progress_sorted(self, sort=1):
        """Returns list of tuples ordered
        by sort"""
        runners = []
        for runner in self.race_runners.values():
            if runner.active and runner.progress and runner.speed:
                runners.append(
                    (runner, runner.progress, runner.progress_seconds_to_finish)
                )
        runners = sorted(runners, key=lambda tup: tup[sort])
        return runners

    @property
    def progress_leader(self):
        progress_sorted = self.progress_sorted()
        if progress_sorted:
            return progress_sorted[0][0]

    @property
    def progress_predicted_winner(self):
        progress_sorted = self.progress_sorted(sort=2)
        if progress_sorted:
            return progress_sorted[0][0]

    """ extra properties """

    @property
    def race_type(self):
        if self._race_type == "Flat":
            return self._race_type
        else:
            return "Jump"

    @property
    def update_flag(self):
        """True if raw data has been updated,
        once per second.
        """
        if self.race_progress:
            ft = self.race_progress.feed_time_epoch
            if str(ft)[-3:] == "000":
                return True

    @property
    def feed_time_equal(self) -> bool:
        """True if all runners data up to date
        with the same feed time epoch"""
        if self.race_progress and self.race_runners:
            fte = [
                r.race_change.feed_time_epoch
                for r in self.race_runners.values()
                if r.active and not r.ignore_runner
            ]
            return fte.count(fte[0]) == len(fte)
        else:
            return False

    def __str__(self):
        return "StatumMarket {0} {1} {2} {3}".format(
            self.race_type, self.surface_type, self.market_id, self.active
        )
