import unittest
from unittest import mock

from statum.market import StatumMarket


class TestStatumInitMarket(unittest.TestCase):
    def setUp(self):
        self.market_id = "1.123456"
        self.course = "Windsor"
        self.race_type = "Flat"
        self.surface_type = "Turf"
        self.race_update = mock.Mock()
        self.race_update.race_progress.running_time = 41
        self.race_update.race_progress.speed = 10
        self.race_update.race_progress.progress = 82
        self.race_update.race_progress.order = [4, 5, 3, 2, 12]
        self.race_update.race_runners = {}
        self.race_distance = 1024
        self.max_latency = 16
        self.min_speed = 2
        self.max_speed = 4
        self.market = StatumMarket(
            self.market_id,
            self.course,
            self.race_type,
            self.surface_type,
            self.race_distance,
            max_latency=self.max_latency,
            min_speed=self.min_speed,
            max_speed=self.max_speed,
        )

    def test_init(self):
        self.assertEqual(self.market.market_id, self.market_id)
        self.assertEqual(self.market.course, self.course)
        self.assertEqual(self.market._race_type, self.race_type)
        self.assertEqual(self.market.surface_type, self.surface_type)
        self.assertEqual(self.market.race_distance, self.race_distance)
        self.assertEqual(self.market.max_latency, self.max_latency)
        self.assertEqual(self.market.min_speed, self.min_speed)
        self.assertEqual(self.market.max_speed, self.max_speed)
        self.assertIsNotNone(self.market._finish_straight_distance)
        self.assertIsNone(self.market.race_progress)
        self.assertEqual(self.market.race_runners, {})
        self.assertTrue(self.market.active)

    def test_init_warning(self):
        assert StatumMarket(self.market_id, "Lingfiel", self.race_type)

    def test_call(self):
        mock_race_update = mock.Mock()
        mock_race_update.race_runners = {}
        mock_market_book = mock.Mock()
        mock_runner = mock.Mock()
        mock_runner.selection_id = 12
        mock_market_book.runners = [mock_runner]
        self.market(mock_race_update, mock_market_book)

        self.assertEqual(self.market.race_progress, mock_race_update.race_progress)
        self.assertEqual(self.market.race_runners, mock_race_update.race_runners)

    def test_call_market_book_dict(self):
        mock_race_update = mock.Mock()
        mock_race_update.race_runners = {}
        mock_market_book = {"runners": [{"selectionId": 21}]}
        self.market(mock_race_update, mock_market_book)

        self.assertEqual(self.market.race_progress, mock_race_update.race_progress)
        self.assertEqual(self.market.race_runners, mock_race_update.race_runners)


class TestStatumMarket(unittest.TestCase):
    def setUp(self):
        self.market_id = "1.123456"
        self.course = "Windsor"
        self.race_type = "Flat"
        self.market = StatumMarket(self.market_id, self.course, self.race_type)

        race_progress = mock.Mock()
        race_progress.running_time = 41
        race_progress.speed = 10
        race_progress.progress = 82
        race_progress.order = [4, 5, 3, 2, 12]
        race_progress.feed_time_epoch = 1522177947000
        self.market.race_progress = race_progress

    def test_race_running_time(self):
        self.assertEqual(self.market.race_running_time, 41)

    def test_race_running_time_left(self):
        self.assertEqual(self.market.race_running_time_left, 8.2)

    def test_race_distance_left(self):
        self.assertEqual(self.market.race_distance_left, 82)

    def test_race_on_finish_straight(self):
        self.market._finish_straight_distance = 540
        self.assertTrue(self.market.race_on_finish_straight)

        self.market._finish_straight_distance = 12
        self.assertFalse(self.market.race_on_finish_straight)

        self.market._finish_straight_distance = 540
        self.market.race_progress.progress = 0
        self.assertFalse(self.market.race_on_finish_straight)

        self.market._finish_straight_distance = None
        self.assertFalse(self.market.race_on_finish_straight)

    def test_runner_distance_to_finish(self):
        mock_runner = mock.Mock(progress=1024)
        self.market.race_runners = {12: mock_runner}
        self.assertEqual(round(self.market.runner_distance_to_finish(12), 5), 1024)

    def test_runner_seconds_to_finish(self):
        mock_runner = mock.Mock(progress_seconds_to_finish=102.4)
        self.market.race_runners = {12: mock_runner}
        self.assertEqual(self.market.runner_seconds_to_finish(12), 102.4)

    def test_runner_position(self):
        self.assertEqual(self.market.runner_position(4), 0)
        self.assertEqual(self.market.runner_position(3), 2)
        self.assertEqual(self.market.runner_position(12), 4)

    def test_runner_position_calculated(self):
        mock_runner_one = mock.Mock()
        mock_runner_one.runner_distance_to_finish = 128
        mock_runner_two = mock.Mock()
        mock_runner_two.runner_distance_to_finish = 156
        self.market.race_runners = {12345: mock_runner_one, 45678: mock_runner_two}
        self.assertEqual(self.market.runner_position_calculated(12345), 0)
        self.assertEqual(self.market.runner_position_calculated(45678), 1)
        self.assertIsNone(self.market.runner_position_calculated(54321))

    def test_progress_sorted(self):
        mock_runner_one = mock.Mock()
        mock_runner_one.progress = 1024
        mock_runner_one.progress_seconds_to_finish = 45.6
        mock_runner_one.active = True
        mock_runner_two = mock.Mock()
        mock_runner_two.progress = 512
        mock_runner_two.progress_seconds_to_finish = 23.2
        mock_runner_two.active = True
        mock_runner_three = mock.Mock()
        mock_runner_three.progress = 994
        mock_runner_three.progress_seconds_to_finish = 47
        mock_runner_three.active = True
        mock_runner_four = mock.Mock()
        mock_runner_four.progress = 2056
        mock_runner_four.progress_seconds_to_finish = 2.12
        mock_runner_four.active = False
        self.market.race_runners = {
            123: mock_runner_one,
            456: mock_runner_two,
            789: mock_runner_three,
            101: mock_runner_four,
        }

        progress_sorted = self.market.progress_sorted()
        self.assertEqual(len(progress_sorted), 3)
        self.assertEqual(progress_sorted[0], (mock_runner_two, 512, 23.2))
        self.assertEqual(progress_sorted[-1], (mock_runner_one, 1024, 45.6))

    def test_progress_sorted_none(self):
        self.market.race_runners = {}

        progress_sorted = self.market.progress_sorted()
        self.assertEqual(len(progress_sorted), 0)

    @mock.patch(
        "statum.market.StatumMarket.progress_sorted",
        return_value=[(4, None), (3, None)],
    )
    def test_progress_leader(self, mock_progress_sorted):
        self.assertEqual(self.market.progress_leader, 4)
        mock_progress_sorted.assert_called_with()

    @mock.patch(
        "statum.market.StatumMarket.progress_sorted",
        return_value=[(3, None), (4, None)],
    )
    def test_progress_predicted_winner(self, mock_progress_sorted):
        self.assertEqual(self.market.progress_predicted_winner, 3)
        mock_progress_sorted.assert_called_with(sort=2)

    def test_race_type(self):
        self.assertEqual(self.market.race_type, self.race_type)
        self.market._race_type = "Bumper"
        self.assertEqual(self.market.race_type, "Jump")

    def test_update_flag(self):
        self.assertTrue(self.market.update_flag)
        self.market.race_progress.feed_time_epoch = 1522177947001
        self.assertIsNone(self.market.update_flag)

    def test_feed_time_equal(self):
        self.market.race_progress = None
        self.assertFalse(self.market.feed_time_equal)

        mock_runner = mock.Mock(active=True, ignore_runner=False)
        self.market.race_runners = {1: mock_runner}
        self.assertFalse(self.market.feed_time_equal)

        self.market.race_progress = mock.Mock()
        mock_runner_one = mock.Mock(active=True, ignore_runner=False)
        mock_runner_one.race_change.feed_time_epoch = 12345
        mock_runner_two = mock.Mock(active=False, ignore_runner=False)
        mock_runner_two.race_change.feed_time_epoch = 123456
        mock_runner_three = mock.Mock(active=True, ignore_runner=False)
        mock_runner_three.race_change.feed_time_epoch = 123456
        mock_runner_four = mock.Mock(active=True, ignore_runner=True)
        mock_runner_four.race_change.feed_time_epoch = 12345678
        self.market.race_runners = {
            1: mock_runner_one,
            2: mock_runner_two,
            3: mock_runner_three,
            4: mock_runner_four,
        }
        self.assertFalse(self.market.feed_time_equal)

        mock_runner_three.race_change.feed_time_epoch = 12345
        self.market.race_runners = {
            1: mock_runner_one,
            2: mock_runner_two,
            3: mock_runner_three,
            4: mock_runner_four,
        }
        self.assertTrue(self.market.feed_time_equal)

    def test_str(self):
        self.assertEqual(str(self.market), "StatumMarket Flat Turf 1.123456 True")
