import unittest
import datetime
from unittest import mock

from statum.runner import StatumRunner


class TestStatumRunner(unittest.TestCase):
    def setUp(self):
        self.race_change = mock.Mock()
        self.race_change.selection_id = 4312
        self.race_change.lat = 12
        self.race_change.long = -12
        self.race_change.speed = 7
        self.runner = StatumRunner(self.race_change, 5, 40)

    def test_init(self):
        self.assertEqual(self.runner.race_change, self.race_change)
        self.assertTrue(self.runner._active)
        self.assertFalse(self.runner._removed)
        self.assertIsNotNone(self.runner.position)

    def test_check_active(self):
        self.runner.check_active()
        self.assertTrue(self.runner._active)

        self.race_change.speed = 4
        self.runner.check_active()
        self.assertFalse(self.runner._active)

        self.race_change.speed = 23
        self.runner.check_active()
        self.assertTrue(self.runner._active)

        self.race_change.speed = 41
        self.runner.check_active()
        self.assertFalse(self.runner._active)

        self.race_change.speed = 25
        self.runner.check_active()
        self.assertTrue(self.runner._active)

    def test_process_runner_book_dict(self):
        runner_book = {"status": "ACTIVE", "ex": {"availableToBack": []}}
        self.runner._process_runner_book(runner_book)
        self.assertFalse(self.runner._removed)
        self.assertEqual(self.runner.runner_book, runner_book)
        runner_book = {"status": "REMOVED", "ex": {"availableToBack": []}}
        self.runner._process_runner_book(runner_book)
        self.assertTrue(self.runner._removed)
        self.assertEqual(self.runner.runner_book, runner_book)

        runner_book = {"status": "ACTIVE", "ex": {"availableToBack": [{"price": 900}]}}
        self.runner._process_runner_book(runner_book)
        self.assertFalse(self.runner._price_removed)
        self.assertEqual(self.runner.runner_book, runner_book)
        runner_book = {
            "status": "REMOVED",
            "ex": {"availableToBack": [{"price": 1000}]},
        }
        self.runner._process_runner_book(runner_book)
        self.assertTrue(self.runner._price_removed)
        self.assertEqual(self.runner.runner_book, runner_book)

    def test_process_runner_book_object(self):
        mock_runner_book = mock.Mock()
        mock_runner_book.status = "ACTIVE"
        mock_price_size = mock.Mock()
        mock_price_size.price = 900
        mock_runner_book.ex.available_to_back = [mock_price_size]
        self.runner._process_runner_book(mock_runner_book)
        self.assertFalse(self.runner._removed)
        self.assertEqual(self.runner.runner_book, mock_runner_book)
        mock_runner_book.status = "REMOVED"
        self.runner._process_runner_book(mock_runner_book)
        self.assertTrue(self.runner._removed)
        self.assertEqual(self.runner.runner_book, mock_runner_book)

        self.runner._process_runner_book(mock_runner_book)
        self.assertFalse(self.runner._price_removed)
        self.assertEqual(self.runner.runner_book, mock_runner_book)
        mock_price_size.price = 1000
        self.runner._process_runner_book(mock_runner_book)
        self.assertTrue(self.runner._price_removed)
        self.assertEqual(self.runner.runner_book, mock_runner_book)

    def test_process_runner_book_flumine(self):
        mock_runner_book = mock.Mock()
        mock_runner_book.status = "ACTIVE"
        mock_price_size = mock.Mock()
        mock_price_size.price = 900
        mock_runner_book.ex.available_to_back = [{"price": 2.56}]  # flumine.patching.py
        self.runner._process_runner_book(mock_runner_book)
        self.assertFalse(self.runner._removed)
        self.assertEqual(self.runner.runner_book, mock_runner_book)

    def test_progress(self):
        self.race_change.progress = 2048
        self.assertEqual(self.runner.progress, 2048)
        self.race_change.progress = None
        self.assertIsNone(self.runner.progress)

    def test_progress_seconds_to_finish(self):
        self.race_change.progress = 2048
        self.race_change.speed = 25.6
        self.assertEqual(self.runner.progress_seconds_to_finish, 80.0)

    def test_progress_seconds_to_finish_zero(self):
        self.race_change.progress = 2048
        self.race_change.speed = 0
        self.assertIsNone(self.runner.progress_seconds_to_finish)

    def test_speed(self):
        self.race_change.speed = 25.6
        self.assertEqual(self.runner.speed, 25.6)
        self.race_change.speed = None
        self.assertIsNone(self.runner.speed)

    def test_stride_frequency(self):
        self.race_change.stride_frequency = 1.28
        self.assertEqual(self.runner.stride_frequency, 1.28)
        self.race_change.stride_frequency = None
        self.assertIsNone(self.runner.stride_frequency)

    def test_price_removed(self):
        self.assertFalse(self.runner.ignore_runner)
        self.runner._price_removed = True
        self.assertTrue(self.runner.ignore_runner)

    def test_active(self):
        self.assertTrue(self.runner.active)
        self.runner._active = False
        self.assertFalse(self.runner.active)
        self.runner._active = False
        self.runner._removed = True
        self.assertFalse(self.runner.active)

    def test_feed_latency(self):
        self.race_change.feed_time = datetime.datetime.utcnow()
        self.assertGreater(self.runner.feed_latency, 0)

    def test_selection_id(self):
        self.assertEqual(self.runner.selection_id, 4312)

    def test_str(self):
        self.assertEqual(str(self.runner), "StatumRunner 4312 True")
